#! /usr/bin/env python

"""List filenames cotaining unfriendly characters."""

"""
Copyright 2006 Edward C. Jones

This file is part of mrquery.

mrquery is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

mrquery is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
mrquery; if not, write to the Free Software Foundation, Inc., 51 Franklin
St, Fifth Floor, Boston, MA 02110-1301 USA
"""
import sys, os, re, optparse

# UNIX filenames can contain characters that cause trouble for many
# programs. List any crummy filenames.

def filename_check(topdirs, f):
    """Find any filenames containing characters other than "a-zA-Z0-9/.-_+".

    Do not check names in  UNIX hidden directories.
    """
    if isinstance(topdirs, str):
        topdirs = (topdirs,)
    badchars = re.compile('[^\w/.\-+]')
    count = 0
    for topdir in topdirs:
        if not os.path.exists(topdir):
            print >> sys.stderr, topdir, 'is not a directory.'
        for dirpath, dirnames, filenames in os.walk(topdir):
            for filename in filenames:
                fullname = os.path.join(dirpath, filename)
                if badchars.search(fullname):
                    f.write('%s\n' % fullname)
                if count > 0 and count % 5000 == 0:
                    print count
                count += 1

            # Exclude UNIX hidden directories.
            hidden = list()
            for i, dirname in enumerate(dirnames):
                if dirname.startswith('.'):
                    hidden.append(i)
            hidden.reverse()
            for i in hidden:
                del dirnames[i]
    print 'Total count is', count

if __name__ == '__main__':
    usage = 'USAGE: ./filename_check.py [-h] <topdir> [<topdir> ...]'
    parser = optparse.OptionParser(usage=usage)
    options, args = parser.parse_args()
    if len(args) == 0:
        parser.print_help()
        sys.exit('one or more topdirs required')
    filename_check(args, sys.stdout)
