#! /usr/bin/env python

"""Converts the output of "getXtext.py" to a set of "rm" statements"""

"""
Copyright 2006 Edward C. Jones

This file is part of mrquery.

mrquery is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

mrquery is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
mrquery; if not, write to the Free Software Foundation, Inc., 51 Franklin
St, Fifth Floor, Boston, MA 02110-1301 USA
"""
import sys, optparse
from LN import LN, MLLN

def getname(line):
    isXed = line.startswith('X ')
    line = line[1:].strip()
    iden_code, num, score, name = line.split()
    is_iden = (iden_code == 'I')
    return name, isXed, is_iden

usage = 'USAGE: ./munge_output.py <input>'
parser = optparse.OptionParser(usage=usage)
options, args = parser.parse_args()

if len(args) != 1:
    parser.print_help()
    sys.exit(2)
filein = args[0]

text = open(filein, 'r').read().strip()
out = list()
text = text.replace('\n\n\n', '\n\n')
text = text.strip()
parts = text.split('\n|')
for part in parts:
    lines = part.splitlines()
    """The first six lines look like:
        | 10 | carltree | carltree | 00010 |
        X Keep all images
          Delete first image in each pair of identical images.
          Delete second image in each pair of identical images.
          Delete first image of each non-identical pair.
          Delete second image of each non-identical pair.
    """
    out.append('# ' + lines[0])
    words = lines[1].split()
    keepall = (words[0] == 'X' and words[1] == 'Keep')

    words = lines[2].split()
    kill_first_of_each_iden_pair = (words[0] == 'X' and words[1] == 'Delete')
    words = lines[3].split()
    kill_second_of_each_iden_pair = (words[0] == 'X' and words[1] == 'Delete')
    words = lines[4].split()
    kill_first_of_each_non_iden_pair = (words[0] == 'X' and words[1] == 'Delete')
    words = lines[5].split()
    kill_second_of_each_non_iden_pair = (words[0] == 'X' and words[1] == 'Delete')
    del lines[:7]

    if kill_first_of_each_iden_pair and kill_second_of_each_iden_pair:
        raise TypeError('Trying to delete all the identical images')
    if kill_first_of_each_non_iden_pair and kill_second_of_each_non_iden_pair:
        raise TypeError('Trying to delete all the pairs '
                   'of non-identical images')

    if keepall:
        LN('"Keep all images" is X\'ed. Nothing to do.')
        sys.exit()
    else:
        for i in range(0, len(lines), 3):
            group = lines[i:i+3]
            if len(group) == 3 and len(group[2]) != 0:
                LN(group)
                raise TypeError('BUG or file format problem')

            name0, isXed0, is_iden = getname(group[0])
            name1, isXed1, is_iden = getname(group[1])
            if isXed0 and isXed1:
                raise ValueError('trying to delete both names in a name pair')
            if is_iden:
                if kill_first_of_each_iden_pair:
                    out.append( 'rm ' + name0)
                elif kill_second_of_each_iden_pair:
                    out.append( 'rm ' + name1)
                elif isXed0:
                    out.append( 'rm ' + name0)
                elif isXed1:
                    out.append( 'rm ' + name1)
            else:
                if kill_first_of_each_non_iden_pair:
                    out.append( 'rm ' + name0)
                elif kill_second_of_each_non_iden_pair:
                    out.append( 'rm ' + name1)
                elif isXed0:
                    out.append( 'rm ' + name0)
                elif isXed1:
                    out.append( 'rm ' + name1)

    text = '\n'.join(out)
    print text
