#! /usr/bin/env python

import pickle

class Name(str):
    def __new__(cls, s, mtime):
        self = str.__new__(cls, s)
        self.mtime = mtime
        return self

    # Needed for pickling because "__new__" is used. When unpickling,
    # the output of "__getnewargs__" is fed to "__new__". One is tempted
    # to use "str(self)" where I have used "str.__str__(self)". But
    # "str()" is overridden by this class.
    def __getnewargs__(self):
        return (str.__str__(self), self.mtime)

    def __str__(self):
        return 'Name(%s, %s)' % (str.__str__(self), self.mtime)

    def __repr__(self):
        return 'Name(%s, %s)' % (str.__repr__(self), self.mtime)

    def as_str(self):
        return str.__str__(self)

N = Name('abc', 7)
print type(N), str(N), repr(N), str.__str__(N), N.mtime
s = pickle.dumps(N, pickle.HIGHEST_PROTOCOL)
N2 = pickle.loads(s)
print type(N2), str(N2), repr(N2), str.__str__(N2), N2.mtime
print N == N2, id(N) == id(N2)

N.mtime = 11
print N
