#include "common.h"

/* Copyright 2006 Edward C. Jones

   This file is part of mrquery.

   mrquery is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your option)
   any later version.

   mrquery is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along
   with mrquery; if not, write to the Free Software Foundation, Inc., 51
   Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

Int32 arrIntersection[40];

/*=== inner_Scorearr =======================================*/

static Float64
getweight(Int32* rcarr, int rcarr_size, int layer, PyArrayObject* wtsarr)
{
    Float64 wt = 0.0;
    Int32 i;
    Int32* p = rcarr;
    div_t dt;

    for (i=0; i<rcarr_size; i++) {
        dt = div(*(p++), HAARSIZE);
        wt += PIX3(wtsarr, layer, abs(dt.quot), abs(dt.rem), Float64);
    }
    return wt;
}

static Float64
getweight_bw(int* rcarr, int rcarr_size, PyArrayObject* wtsarr0)
{
    Float64 wt = 0.0;
    Int32 i;
    Int32* p = rcarr;
    div_t dt;

    for (i=0; i<rcarr_size; i++) {
        dt = div(*(p++), HAARSIZE);
        wt += PIX2(wtsarr0, abs(dt.quot), abs(dt.rem), Float64);
    }
    return wt;
}

/* It is assumed that arr1 and arr2 contain no duplicate ints and are
   sorted. The union of arr1 and arr2 is put in arrout. The size of the
   union is returned.

   Find the list that has the smaller first entry, remove that entry, and
   append it to arrout. If the two lists have the same first entry, remove
   both, then append the entry to arrout.

   Note how easy it would be to program any binary logical operation.
*/

/*
static int
Union(Int32* arr1, Int32* arr2, Int32* arrout, int size1, int size2)
{
    int ctr = 0, ctr1 = 0, ctr2 = 0;
    while (ctr1 < size1 && ctr2 < size2) {
        if (arr1[ctr1] < arr2[ctr2]) {
            arrout[ctr++] = arr1[ctr1++];
        } else if (arr2[ctr2] < arr1[ctr1]) {
            arrout[ctr++] = arr2[ctr2++];
        } else {
            arrout[ctr++] = arr1[ctr1++];
            ctr2++;
        }
    }
    while (ctr1 < size1)
        arrout[ctr++] = arr1[ctr1++];
    while (ctr2 < size2)
        arrout[ctr++] = arr2[ctr2++];
    return ctr;
}
*/

/* It is assumed that arr1 and arr2 contain no duplicate ints and are
   sorted. The intersection of arr1 and arr2 is put in arrout. The size
   of the intersection is returned.

   Find the list that has the smaller first entry, remove that entry. If
   the two lists have the same first entry, remove it from both lists, and
   append it to arrout.

   Note how easy it would be to program any binary logical operation.
*/
static int
Intersection(Int32* arr1, Int32* arr2, Int32* arrout, int size1, int size2)
{
    int ctr = 0, ctr1 = 0, ctr2 = 0;
    while (ctr1 < size1 && ctr2 < size2) {
        if (arr1[ctr1] < arr2[ctr2]) {
            ctr1++;
        } else if (arr2[ctr2] < arr1[ctr1]) {
            ctr2++;
        } else {
            arrout[ctr++] = arr1[ctr1++];
            ctr2++;
        }
    }
    return ctr;
}

static Float64
compare_pair(int r, int c, int usewts, PyArrayObject* bwbits,
         PyArrayObject* Darr, PyArrayObject* wtsarr, PyArrayObject* wtsarr0)
{
    Float64 rwt, cwt;
    Float64 Iscore;
    int Isize;
    Int32* rptrDarr = NULL;
    Int32* cptrDarr = NULL;

    if (PIX1(bwbits, r, Bool) || PIX1(bwbits, c, Bool)) {
        /* Black and white image. */
        /* Pointers to elemtnts of Darr. */
        rptrDarr = &PIX3(Darr, 0, r, 0, Int32);
        cptrDarr = &PIX3(Darr, 0, c, 0, Int32);
        Isize = Intersection(rptrDarr, cptrDarr, arrIntersection,
                  RCQUOTA, RCQUOTA);
        if (usewts) {
            rwt = getweight_bw(rptrDarr, RCQUOTA, wtsarr0);
            cwt = getweight_bw(cptrDarr, RCQUOTA, wtsarr0);
            Iscore = getweight_bw(arrIntersection, Isize, wtsarr0);
        } else {
            rwt = RCQUOTA;
            cwt = RCQUOTA;
            Iscore = (Float64) Isize;
        }
        // L2 normalization. See L2norm.txt for the math.
        return Iscore / sqrt(rwt * cwt);
    } else {
        /* Color image. */
        int layer;
        double Isum = 0.0, Rsum = 0.0, Csum = 0.0;

        if (!usewts) {
            Rsum = LAYERS * RCQUOTA;
            Csum = LAYERS * RCQUOTA;
        }
        for (layer=0; layer<LAYERS; layer++) {
            rptrDarr = &PIX3(Darr, layer, r, 0, Int32);
            cptrDarr = &PIX3(Darr, layer, c, 0, Int32);
            Isize = Intersection(rptrDarr, cptrDarr, arrIntersection,
                             RCQUOTA, RCQUOTA);
            if (usewts) {
                Rsum += getweight(rptrDarr, RCQUOTA, layer, wtsarr);
                Csum += getweight(cptrDarr, RCQUOTA, layer, wtsarr);
                Isum += getweight(arrIntersection, Isize, layer, wtsarr);
            } else {
                Isum += (double) Isize;
            }
        }
        // L2 normalization.
        return Isum / sqrt(Rsum * Csum);
    }
}

int
Cinner_Scorearr(int rstart, int cstart, int rsize,
         int colblocksize, int usewts, Float64 cutoff,
         PyArrayObject* bad, PyArrayObject* bwbits,
         PyArrayObject* Darr, PyArrayObject* wtsarr,
         PyArrayObject* wtarr0, PyArrayObject* scorearr)
{
    int r, c, chi, ctr = 0;
    Float64 L2score;

    /* The time taken is O(rows*cols). */
    for (r=rstart; r<rstart+rsize; r++) {
        if (PIX1(bad, r, Bool))
            continue;
        chi = max(cstart, min(cstart + colblocksize, r));
        for (c=cstart; c<chi; c++) {
            if (PIX1(bad, c, Bool))
                continue;
            L2score = compare_pair(r, c, usewts, bwbits, Darr, wtsarr, wtarr0);
            if (L2score >= cutoff) {
                PIX2(scorearr, ctr, 0, Float64) = L2score;
                PIX2(scorearr, ctr, 1, Float64) = r;
                PIX2(scorearr, ctr, 2, Float64) = c;
                ctr++;
            }
        }
    }
    return ctr;
}

/* Python wrapper code for "inner_scorearr". */
PyObject*
inner_Scorearr(PyObject* obj, PyObject* args)
{
    int rstart, cstart, rsize, colblocksize, usewts, ctr;
    Float64 cutoff;
    PyObject* obad = NULL;
    PyObject* obwbits = NULL;
    PyObject* oDarr = NULL;
    PyObject* owtsarr = NULL;
    PyObject* owtsarr0 = NULL;
    PyObject* oscorearr = NULL;
    PyArrayObject* bad = NULL;
    PyArrayObject* bwbits = NULL;
    PyArrayObject* Darr = NULL;
    PyArrayObject* wtsarr = NULL;
    PyArrayObject* wtsarr0 = NULL;
    PyArrayObject* scorearr = NULL;

    if (!PyArg_ParseTuple(args, "iiiiidOOOOOO|inner_Scorearr",
            &rstart, &cstart, &rsize, &colblocksize, &usewts,
            &cutoff, &obad, &obwbits, &oDarr, &owtsarr, &owtsarr0,
            &oscorearr))
        goto fail;

    bad = NA_InputArray(obad, tBool, NUM_C_ARRAY);
    if (bad == NULL) goto fail;
    bwbits = NA_InputArray(obwbits, tBool, NUM_C_ARRAY);
    if (bwbits == NULL) goto fail;
    Darr = NA_InputArray(oDarr, tInt32, NUM_C_ARRAY);
    if (Darr == NULL) goto fail;
    wtsarr = NA_InputArray(owtsarr, tFloat64, NUM_C_ARRAY);
    if (wtsarr == NULL) goto fail;
    wtsarr0 = NA_InputArray(owtsarr0, tFloat64, NUM_C_ARRAY);
    if (wtsarr0 == NULL) goto fail;
    scorearr = NA_IoArray(oscorearr, tFloat64, NUM_C_ARRAY);
    if (scorearr == NULL) goto fail;

    ctr = Cinner_Scorearr(rstart, cstart, rsize, colblocksize, usewts,
         cutoff, bad, bwbits, Darr, wtsarr, wtsarr0, scorearr);

    Py_XDECREF(scorearr);
    Py_XDECREF(wtsarr0);
    Py_XDECREF(wtsarr);
    Py_XDECREF(Darr);
    Py_XDECREF(bwbits);
    return PyInt_FromLong(ctr);

fail:
    Py_XDECREF(scorearr);
    Py_XDECREF(wtsarr0);
    Py_XDECREF(wtsarr);
    Py_XDECREF(Darr);
    Py_XDECREF(bwbits);
    return NULL;
}

/*=== Boilerplate =============================================*/

/* Boilerplate: (US govenment) The part of a document that is required by
   regulations and which seldom changes.
*/

static PyMethodDef scorec_Methods[] = {
    {"inner_Scorearr", inner_Scorearr, METH_VARARGS, "inner_Scorearr"},
    {NULL, NULL, 0, NULL}   /* sentinel */
};

void initscorec(void)
{
    Py_InitModule("scorec", scorec_Methods);
    import_libnumarray();
}
